/*
 * AvoidBehavior.java
 * 
 * Copyright 2009 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA 94566, U.S.A.
 * www.ridgesoft.com
 * 
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright
 * notice.
 */

import com.ridgesoft.robotics.Localizer;
import com.ridgesoft.robotics.Pose;
import com.ridgesoft.robotics.Navigator;
import com.ridgesoft.robotics.AnalogInput;

public class AvoidBehavior implements Behavior {
    private boolean isActive;
    private Localizer localizer;
    private Navigator navigator;
    private AnalogInput leftRange;
    private AnalogInput rightRange;
    private int threshold;
    private float turnAmount;
    private int holdTime;
    private long holdUntil;
    private float heading;

    public AvoidBehavior(Localizer localizer, Navigator navigator,
            AnalogInput leftRange, AnalogInput rightRange, int threshold,
            float turnAmount, int holdTime) {
        this.localizer = localizer;
        this.navigator = navigator;
        this.leftRange = leftRange;
        this.rightRange = rightRange;
        this.threshold = threshold;
        this.turnAmount = turnAmount;
        this.holdTime = holdTime;
        isActive = false;
        holdUntil = 0;
        heading = 0.0f;
    }

    public boolean poll() {
        boolean wantControl = false;

        int leftValue = leftRange.sample();
        int rightValue = rightRange.sample();

        if ((leftValue > threshold) || (rightValue > threshold))
            wantControl = true;

        if (isActive) {
            if (wantControl) {
                // object ahead, turn away
                Pose pose = localizer.getPose();
                if (leftValue > rightValue)
                    heading = pose.heading - turnAmount;
                else
                    heading = pose.heading + turnAmount;
                navigator.go(heading);
                holdUntil = System.currentTimeMillis() + holdTime;
            }
            else if (System.currentTimeMillis() < holdUntil) {
                // object out of view, continue driving away
                wantControl = true;
                navigator.go(heading);
            }
            else
                navigator.stop();
        }

        return wantControl;
    }

    public void setActive(boolean isActive) {
        this.isActive = isActive;
    }
}
