/*
 * BehaviorArbiter.java
 * 
 * Copyright 2009 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA 94566, U.S.A.
 * www.ridgesoft.com
 * 
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright
 * notice.
 */

public class BehaviorArbiter extends Thread {
    private Behavior[] behaviors;
    private int period;

    public BehaviorArbiter(Behavior[] behaviors, int period) {
        this.behaviors = behaviors;
        this.period = period;
    }

    public void run() {
        try {
            int activeBehavior = -1;
            long nextTime = System.currentTimeMillis() + period;
            while (true) {
                int i = 0;
                while (i < behaviors.length) {
                    if (behaviors[i].poll()) {
                        if (i != activeBehavior) {
                            if (activeBehavior >= 0)
                                behaviors[activeBehavior].setActive(false);
                            activeBehavior = i;
                            behaviors[activeBehavior].setActive(true);
                        }
                        break;
                    }
                    ++i;
                }
                ++i;

                // Poll remaining behaviors
                while (i < behaviors.length)
                    behaviors[i++].poll();

                long currentTime = System.currentTimeMillis();
                long sleepTime = nextTime - currentTime;
                if (sleepTime > 0) {
                    nextTime += period;
                    Thread.sleep(sleepTime);
                }
                else {
                    nextTime = currentTime + period;
                }
            }
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }
}