/*
 * Interact.java
 * 
 * Copyright 2007 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA 94566, U.S.A.
 * www.ridgesoft.com
 * 
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright
 * notice.
 */

import com.ridgesoft.intellibrain.IntelliBrain;
import com.ridgesoft.io.Display;
import com.ridgesoft.robotics.Motor;
import com.ridgesoft.robotics.ContinuousRotationServo;
import com.ridgesoft.io.LED;
import com.ridgesoft.robotics.AnalogInput;
import com.ridgesoft.robotics.PushButton;
import java.util.Random;
import com.ridgesoft.io.Speaker;
import com.ridgesoft.robotics.IrRemote;
import com.ridgesoft.robotics.sensors.SonyIrRemote;

public class Interact {
    public static final int C4 = 262;
    public static final int C4_SHARP = 277;
    public static final int D4 = 294;
    public static final int D4_SHARP = 311;
    public static final int E4 = 330;
    public static final int F4 = 349;
    public static final int F4_SHARP = 370;
    public static final int G4 = 392;
    public static final int G4_SHARP = 415;
    public static final int A4 = 440;
    public static final int A4_SHARP = 466;
    public static final int B4 = 494;
    public static final int C5 = 523;
    public static final int C5_SHARP = 554;
    public static final int D5 = 587;
    public static final int D5_SHARP = 622;
    public static final int E5 = 659;
    public static final int F5 = 698;
    public static final int F5_SHARP = 740;
    public static final int G5 = 784;
    public static final int G5_SHARP = 831;
    public static final int A5 = 880;
    public static final int A5_SHARP = 932;
    public static final int B5 = 988;

    private static final int CHANNEL_UP = 144;
    private static final int CHANNEL_DOWN = 145;
    private static final int VOLUME_UP = 146;
    private static final int VOLUME_DOWN = 147;
    private static final int PLAY = 223;

    private static Display display;
    private static Motor leftMotor;
    private static Motor rightMotor;
    private static LED leftFwdLED;
    private static LED leftRevLED;
    private static LED rightFwdLED;
    private static LED rightRevLED;
    private static AnalogInput thumbwheel;
    private static PushButton startButton;
    private static PushButton stopButton;
    private static Speaker buzzer;
    private static IrRemote irRemote;

    public static void main(String args[]) {
        display = IntelliBrain.getLcdDisplay();
        display.print(0, "Interact");

        leftMotor = new ContinuousRotationServo(IntelliBrain.getServo(1),
                false, 14);
        rightMotor = new ContinuousRotationServo(IntelliBrain.getServo(2),
                true, 14);

        leftFwdLED = IntelliBrain.getUserLed(1);
        leftRevLED = IntelliBrain.getUserLed(2);
        rightFwdLED = IntelliBrain.getUserLed(3);
        rightRevLED = IntelliBrain.getUserLed(4);

        thumbwheel = IntelliBrain.getThumbWheel();

        startButton = IntelliBrain.getStartButton();
        stopButton = IntelliBrain.getStopButton();
        IntelliBrain.setTerminateOnStop(false);

        buzzer = IntelliBrain.getBuzzer();

        irRemote = new SonyIrRemote(IntelliBrain.getIrReceiver());

        int trick = 1;
        int keyCode = -1;
        startButton.waitReleased();
        while (!startButton.isPressed() && (keyCode != PLAY)) {
            keyCode = irRemote.read();

            if (stopButton.isPressed() || (keyCode == CHANNEL_DOWN)) {
                buzzer.beep();
                stopButton.waitReleased();
                trick++;
            }
            else if (keyCode == CHANNEL_UP) {
                buzzer.beep();
                trick--;
            }

            switch (trick) {
            default:
                trick = 1;
            case 1:
                display.print(0, "Maneuver Square");
                break;
            case 2:
                display.print(0, "Dance");
                break;
            case 3:
                display.print(0, "Play Tune");
                break;
            case 0:
                trick = 4;
            case 4:
                display.print(0, "Remote Control");
                break;
            }

            display.print(1, "Thumbwheel: " + thumbwheel.sample());

            if (keyCode != -1) {
                do {
                    try {
                        Thread.sleep(100);
                    }
                    catch (InterruptedException e) {
                    }
                } while (irRemote.read() != -1);
            }
        }

        buzzer.beep();

        IntelliBrain.setTerminateOnStop(true);

        switch (trick) {
        case 1:
            maneuverSquare(thumbwheel.sample() * 5);
            break;
        case 2:
            dance(thumbwheel.sample());
            break;
        case 3:
            playTune(thumbwheel.sample() + 1000);
            break;
        case 4:
            remoteControl();
            break;
        }
    }

    public static void go(int leftPower, int rightPower, int milliseconds) {
        leftMotor.setPower(leftPower);
        rightMotor.setPower(rightPower);

        if (leftPower > 0) {
            leftFwdLED.on();
            leftRevLED.off();
        }
        else if (leftPower < 0) {
            leftFwdLED.off();
            leftRevLED.on();
        }
        else {
            leftFwdLED.off();
            leftRevLED.off();
        }

        if (rightPower > 0) {
            rightFwdLED.on();
            rightRevLED.off();
        }
        else if (rightPower < 0) {
            rightFwdLED.off();
            rightRevLED.on();
        }
        else {
            rightFwdLED.off();
            rightRevLED.off();
        }

        try {
            Thread.sleep(milliseconds);
        }
        catch (InterruptedException e) {
        }
    }

    public static void stop() {
        leftMotor.stop();
        rightMotor.stop();

        leftFwdLED.off();
        leftRevLED.off();
        rightFwdLED.off();
        rightRevLED.off();
    }

    public static void maneuverSquare(int size) {
        for (int i = 0; i < 4; i++) {
            go(16, 16, size);
            go(8, -8, 600);
        }
        stop();
    }

    public static void dance(int seed) {
        Random random = new Random(seed);

        while (true) {
            int leftPower = random.nextInt() % 17;
            int rightPower = random.nextInt() % 17;
            int time = random.nextInt(400) + 100;
            go(leftPower, rightPower, time);
        }
    }

    public static void playTune(int wholeNote) {
        int quarterNote = wholeNote / 4;
        int halfNote = wholeNote / 2;

        // Mary Had a Little Lamb
        buzzer.play(B4, quarterNote);
        buzzer.play(A4, quarterNote);
        buzzer.play(G4, quarterNote);
        buzzer.play(A4, quarterNote);

        buzzer.play(B4, quarterNote);
        buzzer.play(B4, quarterNote);
        buzzer.play(B4, halfNote);

        buzzer.play(A4, quarterNote);
        buzzer.play(A4, quarterNote);
        buzzer.play(A4, halfNote);

        buzzer.play(B4, quarterNote);
        buzzer.play(D5, quarterNote);
        buzzer.play(D5, halfNote);

        buzzer.play(B4, quarterNote);
        buzzer.play(A4, quarterNote);
        buzzer.play(G4, quarterNote);
        buzzer.play(A4, quarterNote);

        buzzer.play(B4, quarterNote);
        buzzer.play(B4, quarterNote);
        buzzer.play(B4, quarterNote);
        buzzer.play(B4, quarterNote);

        buzzer.play(A4, quarterNote);
        buzzer.play(A4, quarterNote);
        buzzer.play(B4, quarterNote);
        buzzer.play(A4, quarterNote);

        buzzer.play(G4, wholeNote);
    }

    public static void remoteControl() {
        while (true) {
            int keyCode = irRemote.read();

            switch (keyCode) {
            case CHANNEL_UP:
                go(Motor.MAX_FORWARD, Motor.MAX_FORWARD, 100);
                break;
            case CHANNEL_DOWN:
                go(Motor.MAX_REVERSE, Motor.MAX_REVERSE, 100);
                break;
            case VOLUME_DOWN:
                go(Motor.MAX_REVERSE, Motor.MAX_FORWARD, 100);
                break;
            case VOLUME_UP:
                go(Motor.MAX_FORWARD, Motor.MAX_REVERSE, 100);
                break;
            default:
                stop();
                break;
            }
        }
    }
}