/*
 * TractorBeam.java
 * 
 * Copyright 2007 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA 94566, U.S.A.
 * www.ridgesoft.com
 * 
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright
 * notice.
 */

import com.ridgesoft.intellibrain.IntelliBrain;
import com.ridgesoft.io.Display;
import com.ridgesoft.robotics.PushButton;
import com.ridgesoft.robotics.Motor;
import com.ridgesoft.robotics.ContinuousRotationServo;
import com.ridgesoft.robotics.SonarRangeFinder;
import com.ridgesoft.robotics.sensors.ParallaxPing;

public class TractorBeam {
    public static void main(String[] args) {
        try {
            Display display = IntelliBrain.getLcdDisplay();
            display.print(0, "Tractor Beam");

            Motor leftMotor = new ContinuousRotationServo(IntelliBrain
                    .getServo(1), false, 14);
            Motor rightMotor = new ContinuousRotationServo(IntelliBrain
                    .getServo(2), true, 14);

            SonarRangeFinder pingSensor = new ParallaxPing(IntelliBrain
                    .getDigitalIO(3));

            PushButton startButton = IntelliBrain.getStartButton();
            startButton.waitReleased();

            boolean go = false;
            while (true) {
                pingSensor.ping();
                Thread.sleep(100);
                float range = pingSensor.getDistanceInches();

                if (go) {
                    if (range > 0.0f && range < 20.0f) {
                        float gain = 5.0f;
                        float error = range - 6.0f;
                        int power = (int) (gain * error);
                        if (power != 0) {
                            leftMotor.setPower(power);
                            rightMotor.setPower(power);
                        }
                        else {
                            leftMotor.stop();
                            rightMotor.stop();
                        }
                    }
                    else {
                        leftMotor.stop();
                        rightMotor.stop();
                    }
                }
                else if (startButton.isPressed()) {
                    go = true;
                }

                if (range > 0.0f)
                    display.print(1,
                            Integer.toString((int) (range + 0.5f)) + '"');
                else
                    display.print(1, "--");
            }
        }
        catch (Throwable t) {
            t.printStackTrace();
        }
    }
}