/*
 * CollectSensorData.java
 * 
 * Copyright 2009 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA  94566, U.S.A.
 * www.ridgesoft.com
 *
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright notice.
 */

import com.ridgesoft.intellibrain.IntelliBrain;
import com.ridgesoft.robotics.AnalogInput;
import com.ridgesoft.robotics.PushButton;
import com.ridgesoft.robotics.Motor;
import com.ridgesoft.robotics.ContinuousRotationServo;

/**
 * This class is an example program that demonstrates collecting data from a
 * wheel sensor.
 */
public class CollectSensorData {
    public static void main(String args[]) {
        Motor motor = new ContinuousRotationServo(IntelliBrain.getServo(1),
                false, 14);

        AnalogInput sensor = IntelliBrain.getAnalogInput(4);

        PushButton startButton = IntelliBrain.getStartButton();
        startButton.waitReleased();

        System.out.println("Press START to\ncollect data");
        startButton.waitPressed();

        motor.setPower(Motor.MAX_FORWARD);
        try {
            Thread.sleep(500);
        }
        catch (InterruptedException e) {
        }
        int[] samples = new int[100];
        long nextTime = System.currentTimeMillis();
        for (int i = 0; i < samples.length; ++i) {
            samples[i] = sensor.sample();
            nextTime += 5;
            try {
                Thread.sleep(nextTime - System.currentTimeMillis());
            }
            catch (InterruptedException e) {
            }
        }
        motor.setPower(Motor.STOP);

        for (int i = 0; i < samples.length; ++i) {
            System.out.println(Integer.toString(i * 5) + '\t' + samples[i]);
        }
    }
}