/*
 * LocalizerTest.java
 * 
 * Copyright 2009 by RidgeSoft, LLC., PO Box 482, Pleasanton, CA  94566, U.S.A.
 * www.ridgesoft.com
 *
 * RidgeSoft grants you the right to use, modify, make derivative works and
 * redistribute this source file provided you do not remove this copyright notice.
 */

import com.ridgesoft.intellibrain.IntelliBrain;
import com.ridgesoft.io.Display;
import com.ridgesoft.robotics.Motor;
import com.ridgesoft.robotics.ContinuousRotationServo;
import com.ridgesoft.robotics.ShaftEncoder;
import com.ridgesoft.robotics.AnalogShaftEncoder;
import com.ridgesoft.robotics.DirectionListener;

public class LocalizerTest {
    private static Motor leftMotor;
    private static Motor rightMotor;

    public static void main(String[] args) {
        Display display = IntelliBrain.getLcdDisplay();
        display.print(0, "Localizer Test");
        display.print(1, "");

        ShaftEncoder leftEncoder = new AnalogShaftEncoder(IntelliBrain
                .getAnalogInput(4), 250, 750, 30, Thread.MAX_PRIORITY);
        ShaftEncoder rightEncoder = new AnalogShaftEncoder(IntelliBrain
                .getAnalogInput(5), 250, 750, 30, Thread.MAX_PRIORITY);

        Localizer localizer = new OdometricLocalizer(leftEncoder, rightEncoder,
                2.65f, 4.55f, 16, Thread.MAX_PRIORITY - 1, 30);

        leftMotor = new ContinuousRotationServo(IntelliBrain.getServo(1),
                false, 14, (DirectionListener) leftEncoder);
        rightMotor = new ContinuousRotationServo(IntelliBrain.getServo(2),
                true, 14, (DirectionListener) rightEncoder);

        forward(5000);
        rotate(500);
        forward(3000);

        Pose pose = localizer.getPose();
        display.print(0, "x: " + (int) pose.x + ", y: " + (int) pose.y);
        display.print(1, "heading: " + (int) Math.toDegrees(pose.heading));

        while (true);
    }

    public static void forward(int time) {
        leftMotor.setPower(Motor.MAX_FORWARD);
        rightMotor.setPower(Motor.MAX_FORWARD);

        try {
            Thread.sleep(time);
        }
        catch (InterruptedException e) {
        }

        leftMotor.stop();
        rightMotor.stop();
    }

    public static void rotate(int time) {
        leftMotor.setPower(-8);
        rightMotor.setPower(8);

        try {
            Thread.sleep(time);
        }
        catch (InterruptedException e) {
        }

        leftMotor.stop();
        rightMotor.stop();
    }
}